// Filename: instruct.cpp  This file contains methods for the generic instruction class.

#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "instruct.h"
#include "iterator.h"

// Constructor.

Instruction::Instruction (void)
{
    next = NULL;
}


// This method validates an integer string, and converts it to an integer.

int Instruction::read_integer (char *input, int& value)
{
int error = FALSE;
int i = 0;
char *temp = NULL;

    value = 0;
    if (!input)
    {
        error = TRUE;
        cout << "ERROR:  Missing second parameter";
    }
    else
    {
        temp = input;

        // If a minus sign is found, advance to next character.

        if (*input == '-')
        {
            i = 1;
            (input)++;
        }

        // Check each digit in string to make sure it is a valid number.
        
        for (; i < strlen(temp); i++)
        {
            if (!isdigit(*input))
            {
                error = TRUE;
                cout << "ERROR:  Second parameter must be an integer";
            }
            (input)++;
        }
        if (!error)
        {
            value = atoi(temp);
        }
    }
    return error;
}


//  This method will check to make sure the assign statement has the correct syntax:
//  ASSIGN(<variable>,<integer>).

int Assign::check_syntax (Variable& vars)
{
int error = FALSE;
int value;
char *operator1 = NULL;
char *operator2 = NULL;


    operator1 = strtok(NULL, ",");
    if (operator1)
    {
        if (vars.is_variable(operator1))
        {
            op1 = (int)*operator1;
            operator2 = strtok(NULL, ")");
            error = read_integer(operator2, value);
            if (!error)
            {
                op2 = value;
            }
        }
        else
        {
            error = TRUE;
            cout << "ERROR: First parameter must be a variable";
        }
    }
    else
    {
        error = TRUE;
        cout << "ERROR:  Missing first parameter";
    }

    return error;
}


// This method will perform the assignment operation.

int Assign::execute(Variable& vars)
{
   vars[op1] = op2;
}


// This method will check to make sure that the increment statement has the correct syntax:
// INC(<variable>).

int Increment::check_syntax (Variable& vars)
{
int error = FALSE;
char *operator1 = NULL;

    operator1 = strtok(NULL, ")");
    if (operator1)
    {
        if (vars.is_variable(operator1))
        {
            op1 = (int)*operator1;
        }
        else
        {
            error = TRUE;
            cout << "ERROR: Parameter must be a variable";
        }
    }
    else
    {
        error = TRUE;
        cout << "ERROR:  Missing parameter";
    }
    return error;
}


// This method will perform the increment operation.

int Increment::execute(Variable& vars)
{
    vars[op1] = vars[op1] + 1;
}


// This method will check to make sure that the add statement has the correct syntax:
// ADD(<variable>,<integer>).

int Add::check_syntax (Variable& vars)
{
int error = FALSE;
int value;
char *operator1 = NULL;
char *operator2 = NULL;

    operator1 = strtok(NULL, ",");
    if (operator1)
    {
        if (vars.is_variable(operator1))
        {
            op1 = (int)*operator1;
            operator2 = strtok(NULL, ")");
            error = read_integer(operator2, value);
            if (!error)
            {
                op2 = value;
            }
        }
        else
        {
            error = TRUE;
            cout << "ERROR: First parameter must be a variable";
        }
    }
    else
    {
        error = TRUE;
        cout << "ERROR:  Missing first parameter";
    }

    return error;
}


// This method will perform the add operation.

int Add::execute(Variable& vars)
{
    vars[op1] = vars[op1] + op2;
}


// This method will check to make sure that the subtract statement has the correct syntax:
// SUB(<variable>,<integer>).

int Subtract::check_syntax (Variable& vars)
{
int error = FALSE;
int value;
char *operator1 = NULL;
char *operator2 = NULL;

    operator1 = strtok(NULL, ",");
    if (operator1)
    {
        if (vars.is_variable(operator1))
        {
            op1 = (int)*operator1;
            operator2 = strtok(NULL, ")");
            error = read_integer(operator2, value);
            if (!error)
            {
                op2 = value;
            }
        }
        else
        {
            error = TRUE;
            cout << "ERROR: First parameter must be a variable";
        }
    }
    else
    {
        error = TRUE;
        cout << "ERROR:  Missing first parameter";
    }

    return error;
}


// This method will perform the subtraction operation.

int Subtract::execute(Variable& vars)
{
    vars[op1] = vars[op1] - op2;
}


// This method will check to make sure that the copy statement has the correct syntax:
// COPY(<variable>,<variable>).

int Copy::check_syntax (Variable& vars)
{
int error = FALSE;
char *operator1 = NULL;
char *operator2 = NULL;

    operator1 = strtok(NULL, ",");
    if (operator1)
    {
        if (vars.is_variable(operator1))
        {
            op1 = (int)*operator1;
            operator2 = strtok(NULL, ")");
            if (vars.is_variable(operator2))
            {
                op2 = (int)*operator2;
            }
            else
            {
                error = TRUE;
                cout << "ERROR: Second parameter must be a variable";
            }
        }
        else
        {
            error = TRUE;
            cout << "ERROR: First parameter must be a variable";
        }
    }
    else
    {
        error = TRUE;
        cout << "ERROR:  Missing first parameter";
    }
    return error;
}


// This method will perform the copy operation.

int Copy::execute(Variable& vars)
{
    vars[op1] = vars[op2];
}


// Loop Constructor.

Loop::Loop (void)
{
    start = end = NULL;
}


// Destructor; deallocates memory used.

Loop::~Loop (void)
{
    free_loop();
}


// This method will return TRUE if the list is empty.

int Loop::is_empty (void)
{
    return (start == NULL);
}


// This method will place an element at the end of the list.

void Loop::add_loop (Instruction *item)
{
    if (is_empty())
    {
        start = end = item;
    }
    else
    {
        end->set_next(item);
        end = item;
    }
}


// This method will free the list.

void Loop::free_loop(void)
{
Instruction *temp;

    while (!is_empty())
    {
        temp = start;
        start = start->get_next();
        delete temp;
    }
}


// This method will execute the loop construct using an Iterator object to
// traverse the list.

int Loop::execute(Variable& vars)
{
int okay = TRUE;
Instruction *lpr;

    Iterator loop1(this);
    while (okay)
        {
        loop1 = 0;
        lpr = loop1++;

        okay = lpr->execute(vars);
        lpr = loop1++;
        while (okay && lpr)
        {
            lpr->execute(vars);
            lpr = loop1++;
        }
    }
}


// This method will check to make sure that the while statement has the correct syntax:
// WHILENE(<variable>,<variable>).

int Whilene::check_syntax (Variable& vars)
{
int error = FALSE;
char *operator1 = NULL;
char *operator2 = NULL;

    operator1 = strtok(NULL, ",");
    if (operator1)
    {
        if (vars.is_variable(operator1))
        {
            op1 = (int)*operator1;
            operator2 = strtok(NULL, ")");
            if (vars.is_variable(operator2))
            {
                op2 = (int)*operator2;
            }
            else
            {
                error = TRUE;
                cout << "ERROR: Second parameter must be a variable";
            }
        }
        else
        {
            error = TRUE;
            cout << "ERROR: First parameter must be a variable";
        }
    }
    else
    {
        error = TRUE;
        cout << "ERROR:  Missing first parameter";
    }
    return error;
}


// This method will check the while condition.

int Whilene::execute(Variable& vars)
{
    if  (vars[op1] != vars[op2])
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

