// Filename: instruct.h   This file contains the declarations for the instruction class.

#ifndef _instruct_h
#define _instruct_h

#include <stdlib.h>
#include "input.h"
#include "variable.h"

#define FALSE       0
#define TRUE        1


// Instruction is the class declaration for the generic instruction; this is an abstract class.

class Instruction {
    public:
        Instruction (void);                                          // Constructor
        virtual ~Instruction (void){};                               // Destructor
        virtual int check_syntax(Variable& vars) = 0;                // Parse instruction
        virtual int execute (Variable& vars) = 0;                    // Execute instruction
        int read_integer (char *input, int& value);                  // Checks integer syntax
        Instruction *get_next (void){return next;}                   // Returns next pointer
        void set_next (Instruction *other) {next = other;}
    private:
        Instruction *next;
};


// Assign is the class declaration for the assignment statement, subclass of Instruction.

class Assign: public Instruction{
    public:
        Assign (void){}                                             // Constructor
        virtual ~Assign (void){};                                   // Destructor
        int check_syntax (Variable& vars);                          // Parse instruction
        int execute(Variable& vars);                                // Execute instruction
    private:
        int op1;                                                    // Variable operator
        int op2;                                                    // Integer operator
};


// Increment is the class declaration for the increment statement, subclass of Instruction.

class Increment: public Instruction{
    public:
        Increment (void){}                                          // Constructor
        virtual ~Increment (void){};                                // Destructor
        int check_syntax (Variable& vars);                          // Parse instruction
        int execute(Variable& vars);                                // Execute instruction
    private:
        int op1;                                                    // Variable operator
};


// Add is the class declaration for the addition statement, subclass of Instruction.

class Add: public Instruction{
    public:
        Add (void){}                                                // Constructor
        virtual ~Add (void){};                                      // Destructor
        int check_syntax (Variable& vars);                          // Parse instruction
        int execute(Variable& vars);                                // Execute instruction
    private:
        int op1;                                                    // Variable operator
        int op2;                                                    // Integer operator
};


// Subtract is the class declaration for the subtraction statement, subclass of Instruction.

class Subtract: public Instruction{
    public:
        Subtract (void){}                                           // Constructor
        virtual ~Subtract (void){};                                 // Destructor
        int check_syntax (Variable& vars);                          // Parse instruction
        int execute(Variable& vars);                                // Execute instruction
    private:
        int op1;                                                    // Variable operator
        int op2;                                                    // Integer operator
};


// Copy is the class declaration for the copy statement, subclass of Instruction.

class Copy: public Instruction{
    public:
        Copy (void){}                                               // Constructor
        virtual ~Copy (void){};                                     // Destructor
        int check_syntax (Variable& vars);                          // Parse instruction
        int execute(Variable& vars);                                // Execute instruction
    private:
        int op1;                                                    // Variable operator
        int op2;                                                    // Variable operator    
};


// Loop is the class declaration for the "loop constructs", subclass of Instruction.

class Loop:  public Instruction{
    public:
        Loop (void);                                                // Constructor
        ~Loop (void);                                               // Destructor
        void add_loop(Instruction *item);                           // Add to loop list
        void free_loop(void);                                       // Free loop list
        int is_empty(void);                                         // Returns whether list is empty
        int check_syntax(Variable& vars) {return 0;}                // Parse instruction
        int execute(Variable& vars);                                // Execute instruction
        Instruction* get_start(void) {return start;}                // Returns the start of the list
    protected:    
        Instruction *start;                                         // Start of loop list
        Instruction *end;                                           // End of loop list
};

// Whilene is the class declaration for the while not equal statement, subclass of Instruction.

class Whilene: public Loop{
    public:
        Whilene (void){}                                            // Constructor
        ~Whilene(void){}                                            // Destructor
        int check_syntax (Variable& vars);                          // Parse instruction
        int execute(Variable& vars);                                // Execute instruction
    private:
        int op1;                                                    // Variable operator
        int op2;                                                    // Variable operator
};

#endif
